package video.pano.pano_rtc

import com.pano.rtc.api.*
import com.pano.rtc.api.model.RtcAudioProfile
import video.pano.pano_rtc.api.IRtcAudioMixingManager
import video.pano.pano_rtc.api.IRtcEngine
import video.pano.pano_rtc.events.RtcEngineEvent
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getAudioChannel
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getAudioDeviceType
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getAudioEqualizationMode
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getAudioProfileQuality
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getAudioReverbMode
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getAudioSampleRate
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getFeedbackType
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getImageFileFormat
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getOptionType
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getVideoFrameRateType
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getVideoProfileType

class RtcEngineManager : FlutterWrapper(), IRtcEngine, IRtcAudioMixingManager {
    private val whiteboardMap = HashMap<RtcWhiteboard, RtcWhiteboardEngine>()
    private var engine: RtcEngine? = null
    var annotationMgr: RtcAnnotationManager? = null
    var networkMgr: RtcNetworkManager? = null
    var videoStreamEngine: RtcVideoStreamEngine? = null
    
    init {
        methodChannel = PanoRtcPlugin.createMethodChannel("pano_rtc/api_engine")
        eventChannel = PanoRtcPlugin.createEventChannel("pano_rtc/events_engine")
        methodChannel.setMethodCallHandler(this)
        eventChannel?.setStreamHandler(this)
    }
    
    override fun onDetachedFromEngine() {
        super.onDetachedFromEngine()
        annotationMgr?.onDetachedFromEngine()
        networkMgr?.onDetachedFromEngine()
        videoStreamEngine?.onDetachedFromEngine()
    }
    
    fun getRtcWhiteboardEngine(): RtcWhiteboardEngine? {
        return whiteboardMap[engine?.whiteboard]
    }

    override fun create(params: Map<String, *>, callback: Callback) {
        val configMap = params["config"] as Map<*, *>
        
        val config = RtcEngineConfig().apply {
            this.context = PanoRtcPlugin.context
            this.appId = configMap["appId"] as String
            this.server = configMap["rtcServer"] as String
            this.videoCodecHwAcceleration = configMap["videoCodecHwAcceleration"] as Boolean
            this.audioScenario = configMap["audioScenario"] as Int
            this.callback = RtcEngineEvent { methodName, data ->
                emit(methodName, data)
            }
        }

        engine = RtcEngine.create(config)
        callback.success(engine != null)
    }

    override fun destroy() {
        RtcEngine.destroy()
        engine = null
    }

    override fun joinChannel(params: Map<String, *>, callback: Callback) {
        val userId = params["userId"] as Int
        val channelId = params["channelId"] as String
        val token = params["token"] as String
        callback.success(engine?.joinChannel(token, channelId, userId.toLong(), RtcChannelConfig()))
    }

    override fun leaveChannel(callback: Callback) {
        callback.success(engine?.leaveChannel())
    }

    override fun startAudio(callback: Callback) {
        callback.success(engine?.startAudio())
    }

    override fun stopAudio(callback: Callback) {
        callback.success(engine?.stopAudio())
    }

    override fun startVideo(params: Map<String, *>, callback: Callback) {
        val view = params["view"] as RtcView
        val configMap = params["config"] as Map<*, *>
        val config = RtcVideoConfig().apply {
            this.profileType = getVideoProfileType(configMap["profileType"] as Int)
            this.sourceMirror = configMap["sourceMirror"] as Boolean
        }
        engine?.setLocalVideoRender(view)
        callback.success(engine?.startVideo(true, config))
    }

    override fun stopVideo(callback: Callback) {
        callback.success(engine?.stopVideo())
    }

    override fun startPreview(params: Map<String, *>, callback: Callback) {
        val configMap = params["config"] as Map<*, *>
        val profileType = getVideoProfileType(configMap["profileType"] as Int)
        callback.success(engine?.startPreview(profileType, true))
    }

    override fun switchCamera(callback: Callback) {
        callback.success(engine?.switchCamera())
    }

    override fun stopPreview(callback: Callback) {
        callback.success(engine?.stopPreview())
    }

    override fun subscribeAudio(params: Map<String, *>, callback: Callback) {
        val userId = params["userId"] as Int
        callback.success(engine?.subscribeAudio(userId.toLong()))
    }

    override fun unsubscribeAudio(params: Map<String, *>, callback: Callback) {
        val userId = params["userId"] as Int
        callback.success(engine?.unsubscribeAudio(userId.toLong()))
    }

    override fun subscribeVideo(params: Map<String, *>, callback: Callback) {
        val userId = params["userId"] as Int
        val configMap = params["config"] as Map<*, *>
        val profileType = getVideoProfileType(configMap["profileType"] as Int)
        val view = params["view"] as RtcView
        engine?.setRemoteVideoRender(userId.toLong(), view)
        callback.success(engine?.subscribeVideo(userId.toLong(), profileType))
    }

    override fun unsubscribeVideo(params: Map<String, *>, callback: Callback) {
        val userId = params["userId"] as Int
        callback.success(engine?.unsubscribeVideo(userId.toLong()))
    }

    override fun muteAudio(callback: Callback) {
        callback.success(engine?.muteAudio())
    }

    override fun unmuteAudio(callback: Callback) {
        callback.success(engine?.unmuteAudio())
    }

    override fun muteVideo(callback: Callback) {
        callback.success(engine?.muteVideo())
    }

    override fun unmuteVideo(callback: Callback) {
        callback.success(engine?.unmuteVideo())
    }

    override fun subscribeScreen(params: Map<String, *>, callback: Callback) {
        val userId = params["userId"] as Int
        callback.success(engine?.subscribeScreen(userId.toLong()))
    }

    override fun unsubscribeScreen(params: Map<String, *>, callback: Callback) {
        val userId = params["userId"] as Int
        callback.success(engine?.unsubscribeScreen(userId.toLong()))
    }

    override fun setLoudspeakerStatus(params: Map<String, *>, callback: Callback) {
        val enable = params["enable"] as Boolean
        callback.success(engine?.setLoudspeakerStatus(enable))
    }

    override fun isLoudspeakerOn(callback: Callback) {
        callback.success(engine?.isLoudspeakerOn)
    }

    override fun setMicrophoneMuteStatus(params: Map<String, *>, callback: Callback) {
        val enable = params["enable"] as Boolean
        callback.success(engine?.setMicrophoneMuteStatus(enable))
    }

    override fun setAudioDeviceVolume(params: Map<String, *>, callback: Callback) {
        val volume = params["volume"] as Int
        val type = params["type"] as Int
        when (getAudioDeviceType(type)) {
            Constants.AudioDeviceType.Record -> {
                callback.success(engine?.setRecordDeviceVolume(volume))
            }
            Constants.AudioDeviceType.Playout -> {
                callback.success(engine?.setPlayoutDeviceVolume(volume))
            }
            else -> {
                callback.failure(Constants.QResult.InvalidArgs.name, "setAudioDeviceVolume Unsupported AudioDeviceType($type)")
            }
        }
    }

    override fun getAudioDeviceVolume(params: Map<String, *>, callback: Callback) {
        val type = params["type"] as Int
        when (getAudioDeviceType(type)) {
            Constants.AudioDeviceType.Record -> {
                callback.success(engine?.recordDeviceVolume)
            }
            Constants.AudioDeviceType.Playout -> {
                callback.success(engine?.playoutDeviceVolume)
            }
            else -> {
                callback.failure(Constants.QResult.InvalidArgs.name, "getAudioDeviceVolume Unsupported AudioDeviceType($type)")
            }
        }
    }

    override fun getRecordingLevel(callback: Callback) {
        callback.success(engine?.recordingLevel)
    }

    override fun getPlayoutLevel(callback: Callback) {
        callback.success(engine?.playoutLevel)
    }

    override fun whiteboardEngine(callback: Callback) {
        val whiteboard = engine?.whiteboard
        if (whiteboardMap.containsKey(whiteboard)) {
            callback.success(whiteboardMap[whiteboard]?.getMethodChannelName())
        }
    }

    override fun switchWhiteboardEngine(params: Map<String, *>, callback: Callback) {
        val whiteboardId = params["whiteboardId"] as String
        val result = engine?.switchWhiteboard(whiteboardId)
        val whiteboard = engine?.whiteboard
        whiteboard?.let { whiteboardMap.put(it, RtcWhiteboardEngine(whiteboardId, whiteboard)) }
        callback.success(result)
    }

    override fun annotationManager(callback: Callback) {
        if (annotationMgr == null) {
            annotationMgr = RtcAnnotationManager(engine?.annotationMgr)
        }
    }

    override fun networkManager(callback: Callback) {
        if (networkMgr == null) {
            networkMgr = RtcNetworkManager(engine?.networkManager)
        }
    }

    override fun videoStreamManager(callback: Callback) {
        if (videoStreamEngine == null) {
            videoStreamEngine = RtcVideoStreamEngine(engine?.videoStreamManager)
        }
    }

    override fun setOption(params: Map<String, *>, callback: Callback) {
        val type = params["type"] as Int

        if (type == 0) { //FaceBeautify

            val optionMap = params["option"] as Map<*, *>
            val enable = optionMap["enable"] as Boolean
            val intensity = optionMap["intensity"] as Float
            engine?.setFaceBeautify(enable)
            engine?.setFaceBeautifyIntensity(intensity)
            callback.success(Constants.QResult.OK)

        } else if (type == 3) { //AudioEqualizationMode

            val option = params["option"] as Int
            callback.success(engine?.setAudioEqualization(getAudioEqualizationMode(option)))

        } else if (type == 4) { //AudioReverbMode

            val option = params["option"] as Int
            callback.success(engine?.setAudioReverb(getAudioReverbMode(option)))

        } else if (type == 5) { //VideoFrameRate

            val option = params["option"] as Int
            callback.success(engine?.setVideoFrameRate(getVideoFrameRateType(option)))

        } else {
            val optionType = getOptionType(type)
            if (optionType == Constants.PanoOptionType.EnableUploadDebugLogs
                    || optionType == Constants.PanoOptionType.EnableUploadAudioDump
                    || optionType == Constants.PanoOptionType.EnableAudioEarMonitoring
                    || optionType == Constants.PanoOptionType.EnableUploadDebugLogsAtFailure
                    || optionType == Constants.PanoOptionType.EnableCpuAdaption) {

                val enable = params["option"] as Boolean
                callback.success(engine?.setOption(optionType, enable))

            } else if (optionType == Constants.PanoOptionType.EnablePanoAudioProfile) {

                val optionMap = params["option"] as Map<*, *>
                val audioProfile = RtcAudioProfile().apply {
                    this.sampleRate = getAudioSampleRate(optionMap["sampleRate"] as Int)
                    this.channel = getAudioChannel(optionMap["channel"] as Int)
                    this.profileQuality = getAudioProfileQuality(optionMap["profileQuality"] as Int)
                }
                callback.success(engine?.setOption(optionType, audioProfile))

            } else {
                callback.failure(Constants.QResult.InvalidArgs.name, "setOption Unsupported type($type)")
            }

        }
    }

    override fun snapshotVideo(params: Map<String, *>, callback: Callback) {
        val outputDir = params["outputDir"] as String
        val userId = params["userId"] as String
        val optionMap = params["option"] as Map<*, *>
        val format = optionMap["format"] as Int
        val mirror = optionMap["mirror"] as Boolean
        val result = engine?.snapshotVideo(outputDir, userId.toLong(),
                RtcSnapshotVideoOption().apply {
                    this.format = getImageFileFormat(format)
                    this.mirror = mirror
                })
        callback.success(result)
    }

    override fun startAudioDumpWithFilePath(params: Map<String, *>, callback: Callback) {
        val maxFileSize = params["maxFileSize"] as Int
        callback.success(engine?.startAudioDump(maxFileSize.toLong()))
    }

    override fun stopAudioDump(callback: Callback) {
        callback.success(engine?.stopAudioDump())
    }

    override fun sendFeedback(params: Map<String, *>, callback: Callback) {
        val infoMap = params["info"] as Map<*, *>
        val info = RtcEngine.FeedbackInfo().apply {
            this.type = getFeedbackType(infoMap["type"] as Int)
            this.productName = infoMap["productName"] as String
            this.description = infoMap["detailDescription"] as String
            this.contact = infoMap["contact"] as String
            this.extraInfo = infoMap["extraInfo"] as String
            this.uploadLogs = infoMap["uploadLogs"] as Boolean
        }
        callback.success(engine?.sendFeedback(info))
    }

    override fun createAudioMixingTask(params: Map<String, *>, callback: Callback) {
        val taskId = params["taskId"] as Int
        val filename = params["filename"] as String
        callback.success(engine?.audioMixingMgr?.createAudioMixingTask(taskId.toLong(), filename))
    }

    override fun destroyAudioMixingTask(params: Map<String, *>, callback: Callback) {
        val taskId = params["taskId"] as Int
        callback.success(engine?.audioMixingMgr?.destroyAudioMixingTask(taskId.toLong()))
    }

    override fun startAudioMixingTask(params: Map<String, *>, callback: Callback) {
        val taskId = params["taskId"] as Int
        val configMap = params["config"] as Map<*, *>
        val config = RtcAudioMixingConfig().apply {
            enablePublish = configMap["enablePublish"] as Boolean
            publishVolume = configMap["publishVolume"] as Int
            enableLoopback = configMap["enableLoopback"] as Boolean
            loopbackVolume = configMap["loopbackVolume"] as Int
            cycle = configMap["cycle"] as Int
            replaceMicrophone = configMap["replaceMicrophone"] as Boolean
        }
        callback.success(engine?.audioMixingMgr?.startAudioMixingTask(taskId.toLong(), config))
    }

    override fun updateAudioMixingTask(params: Map<String, *>, callback: Callback) {
        val taskId = params["taskId"] as Int
        val configMap = params["config"] as Map<*, *>
        val config = RtcAudioMixingConfig().apply {
            enablePublish = configMap["enablePublish"] as Boolean
            publishVolume = configMap["publishVolume"] as Int
            enableLoopback = configMap["enableLoopback"] as Boolean
            loopbackVolume = configMap["loopbackVolume"] as Int
            cycle = configMap["cycle"] as Int
            replaceMicrophone = configMap["replaceMicrophone"] as Boolean
        }
        callback.success(engine?.audioMixingMgr?.updateAudioMixingTask(taskId.toLong(), config))
    }

    override fun stopAudioMixingTask(params: Map<String, *>, callback: Callback) {
        val taskId = params["taskId"] as Int
        callback.success(engine?.audioMixingMgr?.stopAudioMixingTask(taskId.toLong()))
    }

    override fun resumeAudioMixing(params: Map<String, *>, callback: Callback) {
        val taskId = params["taskId"] as Int
        callback.success(engine?.audioMixingMgr?.resumeAudioMixing(taskId.toLong()))
    }

    override fun pauseAudioMixing(params: Map<String, *>, callback: Callback) {
        val taskId = params["taskId"] as Int
        callback.success(engine?.audioMixingMgr?.pauseAudioMixing(taskId.toLong()))
    }

    override fun getAudioMixingDuration(params: Map<String, *>, callback: Callback) {
        val taskId = params["taskId"] as Int
        callback.success(engine?.audioMixingMgr?.getAudioMixingDuration(taskId.toLong()))
    }

    override fun getAudioMixingCurrentTimestamp(params: Map<String, *>, callback: Callback) {
        val taskId = params["taskId"] as Int
        callback.success(engine?.audioMixingMgr?.getCurrentAudioMixingTimestamp(taskId.toLong()))
    }

    override fun seekAudioMixing(params: Map<String, *>, callback: Callback) {
        val taskId = params["taskId"] as Int
        val timestampMs = params["timestampMs"] as Int
        callback.success(engine?.audioMixingMgr?.seekAudioMixing(taskId.toLong(), timestampMs.toLong()))
    }

    override fun getSdkVersion(callback: Callback) {
        callback.success(engine?.sdkVersion)
    }
}
