package video.pano.pano_rtc

import com.pano.rtc.api.RtcWbView
import com.pano.rtc.api.RtcWhiteboard
import video.pano.pano_rtc.api.IRtcWhiteboard
import video.pano.pano_rtc.events.WhiteboardEvent
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getWBClearType
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getWBFillType
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getWBFontStyle
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getWBImageScalingMode
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getWBRoleType
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getWBSnapshotMode
import video.pano.pano_rtc.utils.EnumValueConvert.StaticParams.getWBToolType

class RtcWhiteboardEngine(whiteboardId: String, private val whiteboard: RtcWhiteboard) : FlutterWrapper(), 
        IRtcWhiteboard {
    private val methodChannelName = getWhiteboardMethodChannelName(whiteboardId)
    private val eventChannelName = getWhiteboardEventChannelName(whiteboardId)

    init {
        methodChannel = PanoRtcPlugin.createMethodChannel(methodChannelName)
        eventChannel = PanoRtcPlugin.createEventChannel(eventChannelName)
        methodChannel.setMethodCallHandler(this)
        eventChannel?.setStreamHandler(this)
        whiteboard.setCallback(WhiteboardEvent { methodName, data ->
            emit(methodName, data)
        })
    }

    fun getMethodChannelName(): String {
        return methodChannelName;
    }

    private fun getWhiteboardMethodChannelName(whiteboardId: String): String {
        return "pano_rtc/api_whiteboard_$whiteboardId"
    }

    private fun getWhiteboardEventChannelName(whiteboardId: String): String {
        return "pano_rtc/events_whiteboard_$whiteboardId"
    }

    override fun setRoleType(params: Map<String, *>, callback: Callback) {
        val type = params["type"] as Int
        callback.success(whiteboard.setRoleType(getWBRoleType(type)))
    }

    override fun open(params: Map<String, *>, callback: Callback) {
        val view = params["view"] as RtcWbView
        callback.success(whiteboard.open(view))
    }

    override fun close(callback: Callback) {
        callback.success(whiteboard.close())
    }

    override fun stop(callback: Callback) {
        callback.success(whiteboard.stop())
    }

    override fun getCurrentPageNumber(callback: Callback) {
        callback.success(whiteboard.currentPageNumber)
    }

    override fun getTotalNumberOfPages(callback: Callback) {
        callback.success(whiteboard.totalNumberOfPages)
    }

    override fun getCurrentScaleFactor(callback: Callback) {
        callback.success(whiteboard.currentScaleFactor)
    }

    override fun setCurrentScaleFactor(params: Map<String, *>, callback: Callback) {
        val scale = params["scale"] as Float
        callback.success(whiteboard.setCurrentScaleFactor(scale))
    }

    override fun setBackgroundImage(params: Map<String, *>, callback: Callback) {
        val imgUrl = params["imageUrl"] as String
        callback.success(whiteboard.setBackgroundImage(imgUrl))
    }

    override fun setBackgroundImageWithPage(params: Map<String, *>, callback: Callback) {
        val imgUrl = params["imageUrl"] as String
        val pageNo = params["pageNo"] as Int
        callback.success(whiteboard.setBackgroundImage(imgUrl, pageNo))
    }

    override fun setBackgroundImageScalingMode(params: Map<String, *>, callback: Callback) {
        val mode = params["mode"] as Int
        callback.success(whiteboard.setBackgroundImageScalingMode(getWBImageScalingMode(mode)))
    }

    override fun setToolType(params: Map<String, *>, callback: Callback) {
        val type = params["type"] as Int
        callback.success(whiteboard.setToolType(getWBToolType(type)))
    }

    override fun setLineWidth(params: Map<String, *>, callback: Callback) {
        val width = params["width"] as Int
        callback.success(whiteboard.setLineWidth(width))
    }

    override fun setFillType(params: Map<String, *>, callback: Callback) {
        val type = params["type"] as Int
        callback.success(whiteboard.setFillType(getWBFillType(type)))
    }

    override fun setFillColor(params: Map<String, *>, callback: Callback) {
        val color = params["color"] as Map<*, *>
        val red = color["red"] as Float
        val green = color["green"] as Float
        val blue = color["blue"] as Float
        val alpha = color["alpha"] as Float
        callback.success(whiteboard.setFillColor(red, green, blue, alpha))
    }

    override fun setForegroundColor(params: Map<String, *>, callback: Callback) {
        val color = params["color"] as Map<*, *>
        val red = color["red"] as Float
        val green = color["green"] as Float
        val blue = color["blue"] as Float
        val alpha = color["alpha"] as Float
        callback.success(whiteboard.setForegroundColor(red, green, blue, alpha))
    }

    override fun setBackgroundColor(params: Map<String, *>, callback: Callback) {
        val color = params["color"] as Map<*, *>
        val red = color["red"] as Float
        val green = color["green"] as Float
        val blue = color["blue"] as Float
        val alpha = color["alpha"] as Float
        callback.success(whiteboard.setBackgroundColor(red, green, blue, alpha))
    }

    override fun setFontStyle(params: Map<String, *>, callback: Callback) {
        val style = params["style"] as Int
        callback.success(whiteboard.setFontStyle(getWBFontStyle(style)))
    }

    override fun setFontSize(params: Map<String, *>, callback: Callback) {
        val size = params["size"] as Int
        callback.success(whiteboard.setFontSize(size))
    }

    override fun undo(callback: Callback) {
        callback.success(whiteboard.undo())
    }

    override fun redo(params: Map<String, *>, callback: Callback) {
        callback.success(whiteboard.redo())
    }

    override fun addPage(params: Map<String, *>, callback: Callback) {
        val autoSwitch = params["autoSwitch"] as Boolean
        callback.success(whiteboard.addPage(autoSwitch))
    }

    override fun insertPage(params: Map<String, *>, callback: Callback) {
        val pageNo = params["pageNo"] as Int
        val autoSwitch = params["autoSwitch"] as Boolean
        callback.success(whiteboard.insertPage(pageNo, autoSwitch))
    }

    override fun removePage(params: Map<String, *>, callback: Callback) {
        val pageNo = params["pageNo"] as Int
        val switchNext = params["switchNext"] as Boolean
        callback.success(whiteboard.removePage(pageNo, switchNext))
    }

    override fun gotoPage(params: Map<String, *>, callback: Callback) {
        val pageNo = params["pageNo"] as Int
        callback.success(whiteboard.gotoPage(pageNo))
    }

    override fun nextPage(callback: Callback) {
        callback.success(whiteboard.nextPage())
    }

    override fun prevPage(callback: Callback) {
        callback.success(whiteboard.prevPage())
    }

    @Suppress("UNCHECKED_CAST")
    override fun addBackgroundImages(params: Map<String, *>, callback: Callback) {
        val urls = params["urls"] as List<String>
        callback.success(whiteboard.addBackgroundImages(urls))
    }

    @Suppress("UNCHECKED_CAST")
    override fun createDocWithImages(params: Map<String, *>, callback: Callback) {
        val urls = params["urls"] as List<String>
        callback.success(whiteboard.createDoc(urls))
    }

    override fun createDocWithFilePath(params: Map<String, *>, callback: Callback) {
        val filePath = params["filePath"] as String
        callback.success(whiteboard.createDoc(filePath))
    }

    override fun deleteDoc(params: Map<String, *>, callback: Callback) {
        val fileId = params["fileId"] as String
        callback.success(whiteboard.deleteDoc(fileId))
    }

    override fun switchDoc(params: Map<String, *>, callback: Callback) {
        val fileId = params["fileId"] as String
        callback.success(whiteboard.switchDoc(fileId))
    }

    override fun saveDocToImages(params: Map<String, *>, callback: Callback) {
        val fileId = params["fileId"] as String
        val outputDir = params["outputDir"] as String
        callback.success(whiteboard.saveDocToImages(fileId, outputDir))
    }

    override fun enumerateFiles(callback: Callback) {
        callback.success(whiteboard.enumerateFiles())
    }

    override fun getCurrentFileId(callback: Callback) {
        callback.success(whiteboard.currentFileId)
    }

    override fun clearContents(params: Map<String, *>, callback: Callback) {
        val curPage = params["curPage"] as Boolean
        val type = params["type"] as Int
        callback.success(whiteboard.clearContents(curPage, getWBClearType(type)))
    }

    override fun clearUserContents(params: Map<String, *>, callback: Callback) {
        val userId = params["userId"] as Long
        val curPage = params["curPage"] as Boolean
        val type = params["type"] as Int
        callback.success(whiteboard.clearUserContents(userId, curPage, getWBClearType(type)))
    }

    override fun snapshot(params: Map<String, *>, callback: Callback) {
        val mode = params["mode"] as Int
        val outputDir = params["outputDir"] as String
        callback.success(whiteboard.snapshot(getWBSnapshotMode(mode), outputDir))
    }

    override fun sendMessage(params: Map<String, *>, callback: Callback) {
        val message = params["message"] as ByteArray
        val userId = params["userId"] as Long
        callback.success(whiteboard.sendMessage(userId, message))
    }

    override fun broadcastMessage(params: Map<String, *>, callback: Callback) {
        val message = params["message"] as ByteArray
        callback.success(whiteboard.broadcastMessage(message))
    }

}